class SentryExceptionHandler:
    def __init__(self, client):
        self.client = client

    def __call__(self, request, response, exception):
        data = {
            'request': {
                'url': request.url,
                'method': request.method,
                'query_string': request.query_string,
                'headers': list(request.headers.items()),
                'cookies': list(request.cookies.items()),
                'env': self.clean_environment(request.env)
            },
            'user': {
                # 'id': 'testuser',
                # 'username': 'fkochem',
                'ip_address': self.guess_user_ip_from_request(request)
                # 'email': 'test2@gmail.org'
            }
        }

        self.client.captureException(data=data)

        raise exception

    def clean_environment(self, env):
        """
        Only keep 'REMOTE_ADDR' and environment attributes which begin
        with 'SERVER_'.
        """
        cleaned_env = {}

        for key, value in env.items():
            if key.startswith('SERVER_') or key == 'REMOTE_ADDR':
                cleaned_env[key] = value

        return cleaned_env

    def guess_user_ip_from_request(self, request):
        """
        Return 'X-FORWARDED-FOR' from the request headers if set,
        otherwise return 'REMOTE_ADDR' from the environment. If both are
        not set, return '127.0.0.1'.
        """
        forwarded_for = request.headers.get('X-FORWARDED-FOR', None)
        if forwarded_for:
            return forwarded_for

        remote_addr = request.env.get('REMOTE_ADDR', None)
        if remote_addr:
            return remote_addr

        return '127.0.0.1'



# Copyright (c) 2015 Fabian Kochem


try:
    from setuptools import setup, find_packages, Command
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages, Command

import platform
import os
import subprocess
import sys

from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)

setup(
    name='hug_sentry',
    version='0.0.1',
    author='Fabian Kochem',
    author_email='fabian.kochem@concepts-and-training.de',
    description='Sentry Exception Handler for hug',
    url='https://github.com/conceptsandtraining/hug_sentry',

    # Dependencies
    install_requires=[
        'hug',
        'raven'
    ],
    tests_require=[
        'pytest==2.9.0'
    ],

    cmdclass={
        'test': PyTest
    },
    entry_points={},
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Database',
        'Topic :: Software Development :: Libraries'
    ],
)
